strmap: enable faster clearing and reusing of strmaps

When strmaps are used heavily, such as is done by my new merge-ort algorithm, and strmaps need to be cleared but then re-used (because of e.g. picking multiple commits to cherry-pick, or due to a recursive merge having several different merges while recursing), free-ing and reallocating map->table repeatedly can add up in time, especially since it will likely be reallocated to a much smaller size but the previous merge provides a good guide to the right size to use for the next merge. Introduce strmap_partial_clear() to take advantage of this type of situation; it will act similar to strmap_clear() except that map->table's entries are zeroed instead of map->table being free'd. Making use of this function reduced the cost of clear_or_reinit_internal_opts() by about 20% in mert-ort, and dropped the overall runtime of my rebase testcase by just under 2%. Signed-off-by: Elijah Newren <newren@gmail.com> Signed-off-by: Junio C Hamano <gitster@pobox.com>

Elijah Newren committed Nov 5, 2020 at 00:22 UTC 6ccdfc2a206d8266ad7613999a0d3f6acdf44c89
2 files changed +12
strmap.c
+6
@@ -64,6 +64,12 @@ void strmap_clear(struct strmap *map, int free_values)
64 hashmap_clear(&map->map);
65 }
66
67 +void strmap_partial_clear(struct strmap *map, int free_values)
68 +{
69 + strmap_free_entries_(map, free_values);
70 + hashmap_partial_clear(&map->map);
71 +}
72 +
73 void *strmap_put(struct strmap *map, const char *str, void *data)
74 {
75 struct strmap_entry *entry = find_strmap_entry(map, str);
strmap.h
+6
@@ -42,6 +42,12 @@ void strmap_init_with_options(struct strmap *map,
42 */
43 void strmap_clear(struct strmap *map, int free_values);
44
45 +/*
46 + * Similar to strmap_clear() but leaves map->map->table allocated and
47 + * pre-sized so that subsequent uses won't need as many rehashings.
48 + */
49 +void strmap_partial_clear(struct strmap *map, int free_values);
50 +
51 /*
52 * Insert "str" into the map, pointing to "data".
53 *